/*
 * mem.c
 *
 * Copyright (c) 1995, John D. Kilburg (john@isri.unlv.edu)
 *
 * See copyright.h for more information.
 */
#include "copyright.h"

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Scrollbar.h>

#include "common.h"
#include "WWWP.h"

typedef struct _mem_state
{
  int used, len;
  struct _mem_state *next;
} MemState;

/*
 * WWWGetMem
 */
void *
WWWGetMem(rw, len)
WWWWidget rw;
int len;
{
  MemState *m;
  void *mem;
  unsigned int blen;

  len = (len / sizeof(void *) + 1) * sizeof(void *);

  m = (MemState *)rw->www.memstate;
  if (m == NULL || (m->used + len) > m->len)
  {
    blen = ((len + sizeof(MemState)) / BUFSIZ + 1) * BUFSIZ;
    m = (MemState *)alloc_mem(blen);
    m->len = blen;
    m->used = sizeof(MemState);
    m->next = rw->www.memstate;
    rw->www.memstate = m;
  }

  mem = (void *)m + m->used;
  m->used += len;

  return(mem);
}

/*
 * WWWFreeMem
 */
void
WWWFreeMem(rw)
WWWWidget rw;
{
  MemState *m, *t;

  for (m = rw->www.memstate; m != NULL; )
  {
    t = m;
    m = m->next;
    free_mem(t);
  }

  rw->www.memstate = NULL;

  return;
}
